from __future__ import print_function
import pickle
import os.path
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
import pandas as pd


class access_sheet:
    def __init__(self,
                 SI=None,
                 RN=None):
        """
        Access the google sheet
        Args:
            SI: google Spreadsheet-ID, generated from the link of the google
                spreadsheet saved in google drive.
            RN: Range Name of the google spreadsheet you want to use. Ex-A1:c100

        Returns:
            Parameters required to initialize the google sheet access process.
        """
        if SI==None:
            raise Exception("SPREADSHEET_ID is missing.")
        if RN==None:
            raise Exception("RANGE_NAME is missing.")
        self.SPREADSHEET_ID = SI
        self.RANGE_NAME = RN
        self.SCOPES = ['https://www.googleapis.com/auth/spreadsheets.readonly']# If modifying these scopes, delete the file token.pickle.
        self.values_input = None
        self.service = None
        self.creds = None

    def __call__(self):
        """
        Access the google sheet and return a dataframe.
        Returns:
            Accessed google sheet in dataframe format.
        """
        if os.path.exists('token.pickle'):
            with open('token.pickle', 'rb') as token:
                self.creds = pickle.load(token)

        if not self.creds or not self.creds.valid:
            if self.creds and self.creds.expired and self.creds.refresh_token:
                self.creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(
                    'credentials.json', self.SCOPES) # here enter the name of your downloaded JSON file
                self.creds = flow.run_local_server(port=0)
            with open('token.pickle', 'wb') as token:
                pickle.dump(self.creds, token)
        self.service = build('sheets', 'v4', credentials=self.creds)

        # Call the Sheets API
        self.sheet = self.service.spreadsheets()
        result_input = self.sheet.values().get(spreadsheetId=self.SPREADSHEET_ID,
                                    range=self.RANGE_NAME).execute()
        self.values_input = result_input.get('values', [])
        if not self.values_input and not values_expansion:
            print('No data found.')
        return self.values_input



from GoogleSheetPlot.application import access_sheet
from GoogleSheetPlot.graph import plot_graph
import pandas as pd
class GoogleSheetPlot:
    def __init__(self):
        """
        Initialize GoogleSheetPlot package
        """
        self.df = None
        self.plot = None
    def getDataFrame(self, SSI, SRN):
        """
        Args:
          SI: google Spreadsheet-ID, generated from the link of the google
              spreadsheet saved in google drive.
          RN: Range Name of the google spreadsheet you want to use. Ex-A1:c100
        Returns:
           Accessed google sheet in dataframe format.
        """
        sheet = access_sheet(SSI, SRN)
        self.df = pd.DataFrame(sheet()[1:], columns=sheet()[0])
        return self.df

    def getPlot(self, x=None, y=None, plot='scatter'):
        """
        plot the necessary graph from the dataframed google sheet.
        Args:
            x: The dataframe column to be plotted over the x-axis.
            y: The dataframe column to be plotted over the y-axis.
            plot: The category of a plot for visualization.
        Reurn:
           plot the necessary graph and save the graph the present directory.
        """
        if x==None or x=='':
            raise Exception(" Column-1 is missing.")
        if y==None or y=='':
            raise Exception(" Column-2 is missing.")
        graph = plot_graph(self.df, x, y)
        graph()



"""Setup for the GoogleSheetPlot package."""

import setuptools


with open('README.md') as f:
    README = f.read()

setuptools.setup(
    author="Victor Basu",
    author_email="basu369victor@gmail.com",
    name='GoogleSheetPlot',
    license="MIT",
    description='GoogleSheetPlot is a python package allow the user to select a google sheet from their Google drive and plots a chart with the values on the sheet.',
    version='v0.1.00',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/victor369basu/GoogleSheetPlot',
    download_url = "https://github.com/victor369basu/GoogleSheetPlot/tree/master/dist/GoogleSheetPlot-0.1.0.tar.gz",
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    install_requires=['requests',
                      'pandas',
                      'matplotlib',
                      'google-api-python-client',
                      'google-auth-httplib2',
                      'google-auth-oauthlib',
                      'pickle-mixin'
                     ],
    classifiers=[
        # Trove classifiers
        # (https://pypi.python.org/pypi?%3Aaction=list_classifiers)
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
    ],
)



import matplotlib.pyplot as plt

class plot_graph:
    def __init__(self, dataFrame=None, x=None, y=None, plot='scatter'):
        """
        plot the necessary graph from the dataframed google sheet.
        Args:
            dataFrame: dataframe to be used for visualization.
            x: The dataframe column to be plotted over the x-axis.
            y: The dataframe column to be plotted over the y-axis.
            plot: The category of a plot for visualization.
        Returns:
            Initialized parameter for plotting the necessary graph.
        """
        self.x = x
        self.y = y
        self.col1 = dataFrame[x]
        self.col2 = dataFrame[y]
        self.plot = plot
        self.plt = None
        self.container={
        'scatter',
        'line',
        'bar'
        }

    def __call__(self):
        """
        plot the necessary graph.
        Save the visualized graph in a .png file.
        """
        if self.plot not in self.container:
            raise Exception("Plot category does not exist. Please select among the following: {}".format(self.container))
        plt.figure(figsize=(18,18))
        if self.plot == 'scatter':
            plt.scatter(self.col1,self.col2)
        if self.plot == 'line':
            plt.plot(self.col1,self.col2)
        if self.plot == 'bar':
            plt.bar(self.col1,self.col2)
        plt.xlabel(self.x,fontsize=18)
        plt.ylabel(self.x,fontsize=18)
        plt.title("{}_{}_{}".format(self.x,self.y,self.plot),fontsize=18)
        plt.savefig("{}_{}_{}.png".format(self.x,self.y,self.plot))
        print("Saved Image Successfully.")
        plt.show()
